<?php

/**
 * Implementiert die beroCloud und SNMP Einstellungen
 * @author Miguel Palmer <mp@beronet.com>
 */
class RemoteManagement {

	private		$m_attr;

	private		$post_attr,
			$post_checked = 0;

	private 	$isRequestRejected = 0,
			$reasonRejected = array();

	function __construct($m, $get = array(), $post = array(), $session = array()) {
		$this->m_attr = $m;

		// TODO: maybe set a checkor for an user input instead the 'config-option' checkor
		if (isset($post['submit'])) {
			unset($post['m'], $post['fixChrome'], $post['dummy']);

			$list_post = array(	// GENERAL
						'submit' => 'select',
						// CLOUD
						'cloud_username' => 'config-option', 'cloud_password' => 'password', 'deregister' => 'bool', 'cloud_enable' => 'bool', 'cloudAddress' => 'hostname',
						// KEY
						'supportKey' => 'str-length', 'supportPermanent' => 'bool', 'disconnect_submit' => 'select',
						// PROXY
						'proxyEnabled' => 'bool', 'proxyAddress' => 'hostname', 'proxyPort' => 'int-range', 'proxyType' => 'select', 'proxyUser' => 'config-option', 'proxySecret' => 'password',
						// SNMP
						'snmp_enabled' => 'bool', 'snmptraps' => 'select', 'snmp_syslocation' => 'hostname', 'snmptraps_sink' => 'config-option', 'snmptraps_intervall' => 'int>=0',
						'snmp_community_rw' => 'snmp', 'snmp_community_ro' => 'snmp', 'snmp_community_trap' => 'snmp',
					  );
		
			$list_post_options = array(	
							'submit'    => array('Register', 'Registrieren', 'Save', 'Speichern', 'Connect', 'Verbinden')	, // impossible to generalize with BUTTON_***** without reworking the class
							'disconnect_submit' => array('Disconnect', 'Trennen')		, // impossible to generalize with CLOUD_SUPPORT_DISCONNECT_BUTTON_DIS without reworking the class (and all classes)
							'proxyPort' => array('min' => 1, 'max' => 65535)		,
							'proxyType' => array('CURLPROXY_HTTP', 'CURLPROXY_SOCKS5')	,
							'snmptraps' => array('snmptrap_isdnl2')				,
							'supportKey'=> array('min' => 8, 'max' => 8)
						  );

			$this->post_checked = 1;
			$this->post_attr = Helper::sanitizePost($post, $list_post, $this->post_checked, $this->reasonRejected, $list_post_options);

			$this->isRequestRejected = !$this->post_checked;
		}
	}

	/*
	 * @return string HTML code
	 */
	function _display_tabs ($ba, $simple_class)
	{
		$tabs = '';
		$display_tabs = '';
		if (strlen($simple_class) == 0)
		{
			$tabs =	"\t\t".'<li><a href="#tabs-3">'. CLOUD_PROXY_SETTINGS .'</a></li>' . "\n".
							"\t\t".'<li><a href="#tabs-4">'. SNMP_TABLE_HEADER_DIS .'</a></li>' . "\n";

			$display_tabs =	$this -> _display_tab3($ba) . "\n" .
											$this -> _display_tab4() . "\n";
		}

		$html_str = '<br/>' . "\n" . '<br/>' . "\n" .
								'<script>' . "\n" .
								"\t" . '$(function() {' . "\n" .
								"\t\t" . '$( "#tabs" ).tabs({active:0})' . "\n" .
								"\t" . '});' . "\n" . 
								'</script>' . "\n" .
								"\n\n" . '<!-- BEGIN TABS //-->' . "\n\n" .
								'<div id="tabs" class="">' . "\n" .
								"\t".'<ul>' . "\n" .
								"\t\t".'<li><a href="#tabs-1">'. CLOUD_TABLE_HEADER_DIS .'</a></li>' . "\n".
								"\t\t".'<li><a href="#tabs-2">'. CLOUD_KEY_HEADER_DIS .'</a></li>' . "\n".
								$tabs .
								"\t".'</ul>' . "\n\n" .

								$this -> _display_tab1() . "\n" .
								$this -> _display_tab2() . "\n" .
								$display_tabs . 

								"\n\n" . '<!-- END TABS //-->' . "\n" .
								'</div>' . "\n\n";

		return $html_str;
	}

	/*
	 * @return string HTML code
	 */
	function _display_tab1 ()
	{
		$html_str = "\n\n" . '<!-- BEGIN TAB1 //-->' . "\n\n" .
								'<div id="tabs-1" style="margin-top: 20px;">' . "\n";

		$html_str .= 	$this->cloudRegisterTable() . "\n";
		$html_str .= 	"<br><br>" . "\n";
		$html_str .= 	$this->cloudTable() . "\n";

		$html_str .=	'</div>' . "\n";

		return $html_str;
	}

	/*
	 * @return string HTML code
	 */
	function _display_tab2 () 
	{
		$html_str = "\n\n" . '<!-- BEGIN TAB2 //-->' . "\n\n" .
								'<div id="tabs-2" style="margin-top: 20px;">' . "\n";

		$html_str .= $this->cloudSupportKey() . "\n";

		$html_str .= '</div>' . "\n";

		return $html_str;
	}

	/*
	 * @return string HTML code
	 */
	function _display_tab3 ($ba)
	{
		$html_str = "\n\n" . '<!-- BEGIN TAB3 //-->' . "\n\n" .
								'<div id="tabs-3" style="margin-top: 20px;">' . "\n";

		$html_str .= $this->cloudProxyTable($ba) . "\n";

		$html_str .= '</div>' . "\n";

		return $html_str;
	}

	/*
	 * @return string HTML code
	 */
	function _display_tab4 ()
	{
		$html_str = "\n\n" . '<!-- BEGIN TAB4 //-->' . "\n\n" .
								'<div id="tabs-4" style="margin-top: 20px;">' . "\n";

		$html_str .= $this->snmpTable() ."\n" .
								 "<br>\n" .
								 "<div class='text-center'>\n" .
								 "\t<i>" . REQUIRED_FIELDS . "</i>\n" . 
								 "</div>\n";

		$html_str .= '</div>' . "\n";

		return $html_str;
	}

	/**
	 * Generiert die HTML Seite
	 * @return string HTML code
	 */
	function display() {
		$m = $this->m_attr;
		$name = "REMOTE_MANAGEMENT_TITLE_DIS";
		include('./includes/header.php');

		$ba = new beroAri();
		require_once( '/usr/fallback/beroConf.php' );
		require_once("./includes/lang.php");
		//Save the changes
		$simple_class = '';
		if ($ba->get('gui_mode') == 'simple') {
			$simple_class = 'simple';
		}
		if ($this->post_checked) {
			$ret .= $this->save($ba);
		} else if ($this->isRequestRejected) {
			$ret .= Helper::displayErrorMessage($this->reasonRejected['request'], $this->reasonRejected['checkor'], $this->reasonRejected['key']);
		}

		$ret .= $this -> _display_tabs($ba, $simple_class);
		$ret .= $this -> javascript();

		return $ret;
	}

	/**
	 * @return string Checkbox cloud enable
	 */
	function cloud_enable() {
		require_once( '/usr/fallback/beroConf.php' );
		$bc = new beroConf('root');
		$cloud_enable = $bc->get("root", "cloud_enable");
		$checked = ($cloud_enable == "1") ? "checked" : "";
		ob_start();
		?>
		<input type='hidden' name='cloud_enable' value='0'>
		<input type='checkbox' name='cloud_enable' id='cloud_enable' value='1' <? echo $checked ?>>
		<?php
		return ob_get_clean();
	}

	/**
	 * @return string Textfeld Cloud Adresse
	 */
	function cloud_url() {
		$ba = new beroAri();
		$cloudAddress = $ba->get("cloudAddress");
		
		$bc = new beroConf('root');
		$cloudkey = $bc->get("root", "cloud_key");
		
		$warning = "onclick='return checkCloudAddress();'";
		
		ob_start();
		?>
		<script>
			function checkCloudAddress() {
				<? if (strlen ($cloudkey) == 0) { ?>
					return true;
				<? } ?>
				var c = <? echo ('"' . $cloudAddress . '"'); ?>;
				var n = document.getElementById("cloudAddress").value;
				if (n == c) {
					return true;
				}
				else {
					return confirm(<? echo ('"' . CLOUD_CHANGE_URL_MESSAGE_DIS .'"' );?>);
				}
			}
		</script>
		<input type='text' class='form-control input-sm' name='cloudAddress' id='cloudAddress' value='<? echo $cloudAddress ?>'>
		<br>
		
		<input type='submit' class='btn btn-default' value='<? echo BUTTON_SAVE ?>' name='submit' <? echo $warning ?> >&nbsp;
		<input type='button' class='btn btn-default' value='<? echo BUTTON_LOAD_DEF ?>' onclick='load();'>
		<?php
		return ob_get_clean();
	}

	/**
	 * @return string Checkbox Enable SNMP
	 */
	function snmp_enabled() {
		require_once( '/usr/fallback/beroConf.php' );
		$bc = new beroConf('root');
		$snmp_enable = $bc->get("root", "snmp-enabled");
		$checked = ($snmp_enable == "1") ? "checked" : "";
		ob_start();
		?>
		<input type='hidden' name='snmp_enabled' value='0'>
		<input type='checkbox' id='snmp_enable' onchange='snmp_show();' name='snmp_enabled' value='1' <? echo $checked ?>>
		<?php
		return ob_get_clean();
	}

	/**
	 * @return string Textfeld SNMP Syslocation
	 */
	function snmp_syslocation() {
		require_once( '/usr/fallback/beroConf.php' );
		$bc = new beroConf('root');
		$snmp_syslocation = $bc->get('root', 'snmp-syslocation');
		ob_start();
		?>
		<input type="text" class="form-control input-sm" name="snmp_syslocation" value="<? echo $snmp_syslocation ?>" maxlength="255" >
		<?php
		return ob_get_clean();
	}

	/**
	 * @return string Checkbox Enable SNMP traps
	 */
	function snmptraps() {
		require_once( '/usr/fallback/beroConf.php' );
		$bc = new beroConf('root');
		$snmptraps = $bc->get("root", "snmptraps");
		$snmptraps = explode(",", $snmptraps);
		$checked = (in_array("snmptrap_isdnl2", $snmptraps)) ? "checked" : "";
		ob_start();
		?>
		<input type='checkbox' name='snmptraps' value='snmptrap_isdnl2' <? echo $checked ?>> ISDN-Layer 2
		<?php
		return ob_get_clean();
	}

	/**
	 * @return string Textfeld SNMP traps sink
	 */
	function snmptraps_sink() {
		require_once( '/usr/fallback/beroConf.php' );
		$bc = new beroConf('root');
		$snmptraps_sink = $bc->get("root", "snmptraps_sink");
		ob_start();
		?>
		<input type='text' class='form-control input-sm' name='snmptraps_sink' value='<? echo $snmptraps_sink ?>'>
		<?php
		return ob_get_clean();
	}

	/**
	 * @return string Textfeld SNMP traps intervall
	 */
	function snmptraps_intervall() {
		require_once( '/usr/fallback/beroConf.php' );
		$bc = new beroConf('root');
		$snmptraps_intervall = $bc->get("root", "snmptraps_intervall");
		ob_start();
		?>
		<input type='text' class='form-control input-sm' name='snmptraps_intervall' value='<? echo $snmptraps_intervall ?>'>
		<?php
		return ob_get_clean();
	}

	/**
	 * @return string Textfeld SNMP community ro
	 */
	function ro_community() {
		require_once( '/usr/fallback/beroConf.php' );
		$bc = new beroConf('root');
		$ro_community = $bc->get("root", "snmp-community-ro");
		ob_start();
		?>
		<input type='text' class='form-control input-sm' name='snmp_community_ro' value='<? echo $ro_community ?>'>
		<?php
		return ob_get_clean();
	}

	/**
	 * @return string Textfeld SNMP community rw
	 */
	function rw_community() {
		require_once( '/usr/fallback/beroConf.php' );
		$bc = new beroConf('root');
		$rw_community = $bc->get("root", "snmp-community-rw");
		ob_start();
		?>
		<input type='text' class='form-control input-sm' name='snmp_community_rw' value='<? echo $rw_community ?>'>
		<?php
		return ob_get_clean();
	}

	/**
	 * @return string Textfeld SNMP community trap
	 */
	function community_trap() {
		require_once( '/usr/fallback/beroConf.php' );
		$bc = new beroConf('root');
		$community_trap = $bc->get("root", "snmp-community-trap");
		ob_start();
		?>
		<input type='text' class='form-control input-sm' name='snmp_community_trap' value='<? echo $community_trap ?>'>
		<?php
		return ob_get_clean();
	}

	/**
	 * @return string Checkbox cloud proxy enable
	 */
	function proxy_enable() {
		$ba = new beroAri();
		$proxy_enable = $ba->get("proxyEnabled");

		$checked = ($proxy_enable == "1") ? "checked" : "";
		ob_start();
		?>
		<input type='hidden' name='proxyEnabled' value='0'>
		<input type='checkbox' name='proxyEnabled' id='proxyEnabled' value='1' <? echo $checked ?>>
		<?php
		return ob_get_clean();
	}

	/**
	 * @return string Cloud key
	 */
	function cloud_key() {
		require_once( '/usr/fallback/beroConf.php' );
		$bc = new beroConf('root');
		return $bc->get("root", "cloud_key");
	}

	/**
	 * 
	 * @return string Dropdown cloud proxy type
	 */
	function proxy_type() {
		$ba = new beroAri();
		$proxyType = $ba->get("proxyType");
		$selectedHttp = ($proxyType == "CURLPROXY_HTTP") ? "selected" : "";
		$selectedSocks5 = ($proxyType == "CURLPROXY_SOCKS5") ? "selected" : "";
		ob_start();
		?>
		<select class='form-control input-sm' name='proxyType'>
			<option value='CURLPROXY_HTTP' <? echo $selectedHttp ?>>HTTP</option>
			<option value='CURLPROXY_SOCKS5' <? echo $selectedSocks5 ?>>SOCKS5</option>
		</select>
		<?php
		return ob_get_clean();
	}

	/**
	 * Check, if a valid url is provided
	 * @param string $url
	 * @return boolean
	 */
	function isDomainAvailable($url) {
		$connected = @fsockopen($url, 443); //website and port
		if ($connected) {
			$is_conn = true; //action when connected
			fclose($connected);
		} else {
			$is_conn = false; //action in connection failure
		}
		return $is_conn;
	}

	/**
	 * Generates the javascript code
	 * @return string
	 */
	function javascript() {
		ob_start();
		?>
		<script type="text/javascript">
		    $(document).ready(function () {
		        snmp_show();
		    });
		    function load() {
		        $('#cloudAddress').val('berocloud.beronet.com');
		    }
		    function snmp_show() {
		        if ($('#snmp_enable').is(':checked')) {
		            $('#traps').show();
		            $('#sink').show();
		            $('#interval').show();
		            $('#communityRo').show();
		            $('#communityRw').show();
		            $('#communityTrap').show();
		        } else {
		            $('#traps').hide();
		            $('#sink').hide();
		            $('#interval').hide();
		            $('#communityRo').hide();
		            $('#communityRw').hide();
		            $('#communityTrap').hide();
		        }
		    }
		    $('#cloud').validate({
		        rules: {
		            cloudAddress: {
		                checkHostname: true,
		                required: '#cloud_enable:checked',
		            },
		        }
		    });
		    $('#support').validate({
		        rules: {
		            supportKey: {
		                required: true,
		                minlength: 8,
		                maxlength: 8,
				checkManual: true
		            },
		        }
		    });
		    $('#proxy').validate({
		        rules: {
		            proxyAddress: {
		                required: '#proxyEnabled:checked',
				checkHostname: true
		            },
		            proxyPort: {
		                required: '#proxyEnabled:checked',
		                digits: true,
		                range: [1, 65535],
		            },
			    proxySecret: {
 	    	      		forbidPassword: {
                      			depends: function () {
                      				if ($('#password-dummy').val() == '1') {
                           				return '*****' == $('#proxy-secret').val();
                           			}
                      				return false;
                      			}
                     		}
			    }
		        }
		    });

		    $('#snmp').validate({
		        rules: {
		            snmptraps_intervall: {
		                required: true,
		                digits: true,
		            },
		            snmp_community_ro: {
		                checkSnmp: true
		            },
		            snmp_community_rw: {
		                checkSnmp: true
		            },
		            snmp_community_trap: {
		                checkSnmp: true
		            },
		        }
		    });
		    $('#proxyEnabled').change(function () {
		        required();
		    });
		    $('#cloud_enable').change(function () {
		        required();
		    });
		    function required() {
		        if ($('#proxyEnabled').is(':checked')) {
		            addAsterisk('proxyAddress');
		            addAsterisk('proxyPort');
		        } else {
		            deleteAsterisk('proxyAddress');
		            deleteAsterisk('proxyPort');
		        }
		        if ($('#cloud_enable').is(':checked')) {
		            addAsterisk('cloudAddress');
		        } else {
		            deleteAsterisk('cloudAddress');
		        }
		    }
		    required();
		</script>
		<?php
		return ob_get_clean();
	}

	/**
	 * Generates the form for the cloud registration
	 * @return string HTML code
	 */
	function cloudRegisterTable() {
		ob_start();
		
		$bc = new beroConf('root');
		$cloud_key = $bc->get ("root", "cloud_key");
		
		?>
		<form  class='form-horizontal' action='' method='POST' autocomplete='off'>
			<input name='fixChrome' type='password' style='display:none'>
			<input type=hidden name=m value="<? echo $this->m_attr ?>">
			<div class="form-group">
				<label for='cloud_username' class='col-sm-5 control-label'>Cloud username</label>
				<div class='col-sm-4'>
					<input type='text' class='form-control input-sm' name='cloud_username'>
				</div>
			</div>
			<div class="form-group">
				<label for='cloud_password' class='col-sm-5 control-label'>Cloud password</label>
				<div class='col-sm-4'>
					<input type='password' class='form-control input-sm' autocomplete='new-password' name='cloud_password'>
				</div>
			</div>
			
			<? if (!strlen ($cloud_key)) {?>
			
					<div class="text-center" data-toggle='tooltip' title='<? echo CLOUD_NETWORK_HINT ?>'>
					<input name='submit' class='btn btn-default' type='submit' value='<? echo BUTTON_REGISTER ?>'>
			
			<?	} else { ?>
					<div class="text-center" data-toggle='tooltip' title='<? echo CLOUD_NETWORK_DEREGISTER_HINT ?>'>
					<input name='deregister' class='btn btn-default' type='submit' value='<? echo BUTTON_DEREGISTER ?>' onclick='return confirm("<? echo CLOUD_MESSAGE_DIS ?>?");'>
			<?	} ?>
			</div>
		</form>
		<?php
		
		return ob_get_clean();
	}

	/**
	 * @return string HTML code
	 */
	function cloudSupportKey ()
	{
	
		$ba = new beroAri();
		$permanent_remote_support_active = $ba->get("permanent_remote_support_active");
		$checked = ($permanent_remote_support_active == "1") ? "checked" : "";
		
		ob_start();
		?>
		<form name='support' id='support' action='' class="form-horizontal" method='POST'>
			<div class="form-group">
				<label for='supportKey' class='col-sm-5 control-label'><? echo CLOUD_SUPPORT_KEY_DIS ?></label>
				<div class='col-sm-4'>
					<input type="text" name="supportKey" class='form-control input-sm'>
				</div>
				<label for='supportPermanent' class='col-sm-5 control-label'><? echo CLOUD_SUPPORT_PERMANENT_DIS ?></label>
				<div class='col-sm-4'>
					<input type='checkbox' name='supportPermanent' id='supportPermanent' value='1' <? echo $checked ?>>
				</div>
			</div>
			<div class="text-center">
				<input name='submit' class='btn btn-default' type='submit' value='<? echo BUTTON_CONNECT ?>'>
			</div>
			<br>
		</form>

		<?php
		exec ("ps aux|grep ssh|grep StrictHostKeyChecking=no|grep -v grep", $output, $retval);
		if (sizeof($output) || $permanent_remote_support_active) {
			if (sizeof($output)) {
				$rsstate = CLOUD_SUPPORT_STATE_ACTIVE_INFO;
				$rscolor = "gruen";
			}
			else {
				$rsstate  = CLOUD_SUPPORT_STATE_FAILED_INFO;
				$rscolor = "rot";
			}
		?>
		<form name='disconnect_support' id='disconnect_support' action='' class="form-horizontal" method='POST'>
			<div class="text-center">
				<img src='./includes/images/punkt_<? echo $rscolor ?>.gif'>
				<font color='black'><? echo $rsstate ?><br>
				<input id='disconnect_submit' name='disconnect_submit' class='btn btn-default' type='submit' value=<? echo CLOUD_SUPPORT_DISCONNECT_BUTTON_DIS ?>>
			</div>
			<br>
		</form>
		<?php
		}
		

		return ob_get_clean();		
	}

	/**
	 * @return string HTML form for the general cloud settings
	 */
	function cloudTable() {
		ob_start();
		?>
		<form name='cloud' id='cloud' action='' class="form-horizontal" method='POST'>
			<div class="form-group">
				<label for='cloud_enable' class='col-sm-5 control-label'><? echo CLOUD_ENABLE_DIS ?></label>
				<div class='col-sm-4'>
					<? echo $this->cloud_enable() ?>
				</div>
			</div>
			<div class="form-group">
				<label for='cloudAddress' class='col-sm-5 control-label'><? echo CLOUD_URL_DIS ?></label>
				<div class='col-sm-4'>
					<? echo $this->cloud_url() ?>
				</div>
			</div>
		</form>
		<?php
		return ob_get_clean();
	}

	/**
	 * @param beroAri $ba
	 * @return string HTML form für the cloud proxy settings
	 */
	function cloudProxyTable($ba) {
		ob_start();
		?>
		<form name='proxy' id='proxy' action='' class="form-horizontal" method='POST'>
			<input type='hidden' name='dummy' id='password-dummy' style='display:none' value='0'>
			<div class="form-group">
				<label for='proxyEnabled' class='col-sm-5 control-label'><? echo CLOUD_PROXY_ENABLE ?></label>
				<div class='col-sm-4'>
					<? echo $this->proxy_enable() ?>
				</div>
			</div>
			<div class="form-group">
				<label for='proxyAddress' class='col-sm-5 control-label'><? echo CLOUD_PROXY_ADDRESS ?></label>
				<div class='col-sm-4'>
					<input type='text' class='form-control input-sm' name='proxyAddress' value='<? echo $ba->get('proxyAddress') ?>'>
				</div>
			</div>
			<div class="form-group">
				<label for='proxyPort' class='col-sm-5 control-label'><? echo CLOUD_PROXY_PORT ?></label>
				<div class='col-sm-4'>
					<input type='text' class='form-control input-sm' name='proxyPort' value='<? echo $ba->get('proxyPort') ?>'>
				</div>
			</div>
			<div class="form-group">
				<label for='proxyType' class='col-sm-5 control-label'><? echo CLOUD_PROXY_TYPE ?></label>
				<div class='col-sm-4'>
					<? echo $this->proxy_type() ?>
				</div>
			</div>
			<div class="form-group">
				<label for='proxyUser' class='col-sm-5 control-label'><? echo CLOUD_PROXY_USER ?></label>
				<div class='col-sm-4'>
					<input type='text' class='form-control input-sm' name='proxyUser' value='<? echo $ba->get('proxyUser') ?>'>
				</div>
			</div>
			<div class="form-group">
				<label for='proxySecret' class='col-sm-5 control-label'><? echo CLOUD_PROXY_SECRET ?></label>
				<div class='col-sm-4'>
					<input type='password' class='form-control input-sm' name='proxySecret' id='proxy-secret' onchange="$('#password-dummy').val('1');" value='<? echo strlen($ba->get('proxySecret')) === 0 ? '' : '*****'; ?>'>
				</div>
			</div>
			<div class="text-center">
				<input name='submit' class='btn btn-default' type='submit' value='<? echo BUTTON_SAVE ?>'>
			</div>
			<br>
		</form>
		<?php
		return ob_get_clean();
	}

	/**
	 * Generates the html form for the SNMP settings
	 * @return string HTML form
	 */
	function snmpTable() {
		ob_start();
		?>
		<form id='snmp' action='' method='POST' class="form-horizontal">
			<input type=hidden name=m value="<? echo $this->m_attr ?>">
			<div class="form-group">
				<label for='snmp_enabled' class='col-sm-5 control-label'><? echo SNMP_ENABLED_DIS ?></label>
				<div class='col-sm-4'>
					<? echo $this->snmp_enabled() ?>
				</div>
			</div>
			<div class="form-group" id='snmp_syslocation'>
				<label for='snmp_syslocation' class='col-sm-5 control-label'><? echo SNMP_SYSLOCATION ?></label>
				<div class='col-sm-4'>
					<? echo $this->snmp_syslocation() ?>
				</div>
			</div>
			<div class="form-group" id='traps'>
				<label for='snmptraps' class='col-sm-5 control-label'><? echo SNMP_TRAPS_DIS ?></label>
				<div class='col-sm-4'>
					<? echo $this->snmptraps() ?>
				</div>
			</div>
			<div class="form-group" id='sink'>
				<label for='snmptraps_sink' class='col-sm-5 control-label'><? echo SNMP_TRAPS_SINK_DIS ?></label>
				<div class='col-sm-4'>
					<? echo $this->snmptraps_sink() ?>
				</div>
			</div>
			<div class="form-group" id='interval'>
				<label for='snmptraps_intervall' class='col-sm-5 control-label'><? echo SNMP_TRAPS_INTERVALL_DIS ?></label>
				<div class='col-sm-4'>
					<? echo $this->snmptraps_intervall() ?>
				</div>
			</div>
			<div class="form-group" id='communityRo'>
				<label for='snmp_community_ro' class='col-sm-5 control-label'><? echo SNMP_RO_COMMUNITY_DIS ?></label>
				<div class='col-sm-4'>
					<? echo $this->ro_community() ?>
				</div>
			</div>
			<div class="form-group" id='communityRw'>
				<label for='snmp_community_rw' class='col-sm-5 control-label'><? echo SNMP_RW_COMMUNITY_DIS ?></label>
				<div class='col-sm-4'>
					<? echo $this->rw_community() ?>
				</div>
			</div>
			<div class="form-group" id='communityTrap'>
				<label for='snmp_community_trap' class='col-sm-5 control-label'><? echo SNMP_COMMUNITY_TRAP_DIS ?></label>
				<div class='col-sm-4'>
					<? echo $this->community_trap() ?>
				</div>
			</div>
			<? if (is_file("/usr/local/conf/BEROFIX-MIB.txt")): ?>
				<div class="text-center" >
					<a href="misc/files_download.php?option=BEROFIX-MIB">Download BEROFIX-MIB</a>
				</div>
			<? endif; ?>
			<br>			
			<div class="text-center" >
				<input name='submit' class='btn btn-default' type='submit' value='<? echo BUTTON_SAVE ?>'>
			</div>
		</form>
		<?php
		return ob_get_clean();
	}


	function cloud_deregister ($serial, $ba, $bc) {
			$apiUrl = 'https://' . $ba->get("cloudAddress") . '/index.php?r=api/deregister';
			$ch = curl_init();

			$data = array('serial' => $serial, 'key' => $bc->get('root', 'cloud_key'));

			curl_setopt($ch, CURLOPT_URL, $apiUrl);
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data); // if cloud_key is not crypted => security issue (even for deregistering)
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);

			$result = curl_exec($ch);

			curl_close($ch);
			return "$result";
	}

	/**
	 * Saves the setting in the database
	 * @param beroAri $ba
	 * @return string HTML code
	 */
	function save($ba) {
		require_once( '/usr/fallback/beroConf.php' );
		$bc = new beroConf('root');
		$result = "";
		$serial = Helper::getSerial();

		//we now explicitly have to click the register button to register, only register is performed, no other post stuff will be evaluated
		if (isset($this->post_attr['submit']) && $this->post_attr['submit'] == BUTTON_REGISTER) {
		 	if (! (isset($this->post_attr['cloud_username']) && isset($this->post_attr['cloud_password']) && strlen ($this->post_attr['cloud_username']) && $this->post_attr['cloud_password'])) {
		 		return $table;
		 	}
			
			$apiUrl = 'https://' . $ba->get("cloudAddress") . '/index.php?r=api/register';
			if (($bc->get('root', 'lan-nameserver') == "" || $bc->get('root', 'lan-gateway') == "") && $bc->get('root', 'lan-gateway') == "no")
				$result = CLOUD_NETWORK_HINT;
			else if (!$this->isDomainAvailable($ba->get("cloudAddress")))
				$result = CLOUD_NO_CONNECTION_HINT;
			else {
				$ch = curl_init();

				$data = array('serial' => $serial, 'cloud_username' => $this->post_attr['cloud_username'], 'cloud_password' => $this->post_attr['cloud_password']);

				curl_setopt($ch, CURLOPT_URL, $apiUrl);
				curl_setopt($ch, CURLOPT_POST, 1);
				curl_setopt($ch, CURLOPT_POSTFIELDS, $data); // password send to the cloud without being hashed
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);

				$result = curl_exec($ch);
				preg_match("/[^a-zA-Z0-9=]*?yaqwsx(?<key>.+)yaqwsx/", $result, $key);
				$key = unserialize($key['key']);
				if (isset($key['key'])) {
					$bc->set('root', 'cloud_key', $key['key']);
					$table .= Helper::activate(1, true, $this->m_attr, ACTIVATE);
					$result = preg_replace("/[^a-zA-Z0-9=]*?yaqwsx(?<key>.+)yaqwsx/", ". ", $result);
				}
				curl_close($ch);
			}
			if ($result != "" && strpos($result, "Device was registered"))
				$table .= "<div class='text-center'><img src='./includes/images/punkt_gruen.gif'><font color='black'>" . $result . "<br/></div>";
			elseif ($result != "" && !strpos($result, "Device was registered"))
				$table .= "<div class='text-center'><img src='./includes/images/punkt_rot.gif'><font color='red'>" . $result . "</font><br/></div>";
			return $table;
		}
		
		//we now explicitly have to click the deregister button to register, only deregister is performed, no other post stuff will be evaluated
		if (isset($this->post_attr['deregister'])) {

			$result = $this->cloud_deregister ($serial, $ba, $bc);

			if ($result != "" && trim($result) == "Device was deregistered") {
				$bc->set('root', 'cloud_enable', 0);
				$bc->set('root', 'cloud_key', '');
				$table .= Helper::activate(1, true, $this->m_attr, ACTIVATE);
				$table .= "<div class='text-center'><img src='./includes/images/punkt_gruen.gif'><font color='black'>" . $result . "<br/></div>";
			} elseif ($result != "")
				$table .= "<div class='text-center'><img src='./includes/images/punkt_rot.gif'><font color='red'>" . $result . "</font><br/></div>";
			return $table;
		}
		
		if (isset($this->post_attr['cloud_enable'])) {
			
			$oldAddress = $ba->get('cloudAddress');
			$cloudkey = $bc->get("root", "cloud_key");
			
			//cloud address changed, try to deregister first and reset cloudKey in any case
			if (strcmp ($oldAddress, sqlite_escape_string(trim($this->post_attr['cloudAddress']))) != 0) {

				if (strlen ($cloudkey)) {
					$result = $this->cloud_deregister ($serial, $ba, $bc);
					
					$table .= Helper::activate(1, true, $this->m_attr, ACTIVATE);
					
					if ($result != "" && trim($result) == "Device was deregistered") {
						$table .= "<div class='text-center'><img src='./includes/images/punkt_gruen.gif'><font color='black'>" . $result . "<br/></div>";
					}
					elseif ($result != "") {
						$table .= "<div class='text-center'><img src='./includes/images/punkt_rot.gif'><font color='red'>" . $result . "</font><br/></div>";
					}
				}
				$bc->set('root', 'cloud_key', '');
			}
			
			
			$bc->set('root', 'cloud_enable', sqlite_escape_string($this->post_attr['cloud_enable']));
			$ba->set("cloudAddress", sqlite_escape_string(trim($this->post_attr['cloudAddress'])));

			if (isset($this->post_attr['cloud_key'])) {
				$bc->set('root', 'cloud_key', sqlite_escape_string($this->post_attr['cloud_key']));
			}


			$table .= Helper::activate(1, true, $this->m_attr, ACTIVATE);
			return $table;
		}
		
		if (isset($this->post_attr['snmp_enabled'])) {
			$bc->set('root', 'snmp-enabled', $this->post_attr['snmp_enabled']);
			if ($this->post_attr['snmp_enabled'] == 1) {
				$bc->set('root', 'snmptraps', sqlite_escape_string($this->post_attr['snmptraps']));
				$bc->set('root', 'snmp-syslocation', sqlite_escape_string($this->post_attr['snmp_syslocation']));
				$bc->set('root', 'snmptraps_sink', sqlite_escape_string($this->post_attr['snmptraps_sink']));
				$bc->set('root', 'snmptraps_intervall', sqlite_escape_string($this->post_attr['snmptraps_intervall']));
				$bc->set('root', 'snmp-community-rw', sqlite_escape_string($this->post_attr['snmp_community_rw']));
				$bc->set('root', 'snmp-community-ro', sqlite_escape_string($this->post_attr['snmp_community_ro']));
				$bc->set('root', 'snmp-community-trap', sqlite_escape_string($this->post_attr['snmp_community_trap']));
			} else {
				$bc->set('root', 'snmptraps', "");
				$bc->set('root', 'snmp-syslocation', '');
				$bc->set('root', 'snmptraps_sink', "");
				$bc->set('root', 'snmptraps_intervall', "");
				$bc->set('root', 'snmp-community-rw', "");
				$bc->set('root', 'snmp-community-ro', "");
				$bc->set('root', 'snmp-community-trap', "");
			}

			$table .= Helper::activate(2, true, $this->m_attr, ACTIVATE);
			return $table;
		}
		
		if ($this->post_attr['proxyEnabled'] == '1' && isset($this->post_attr['proxyAddress']) && isset($this->post_attr['proxyPort']) && isset($this->post_attr['proxyType'])) {
			$ba->set('proxyEnabled', sqlite_escape_string($this->post_attr['proxyEnabled']));
			$ba->set('proxyAddress', sqlite_escape_string($this->post_attr['proxyAddress']));
			$ba->set('proxyPort', sqlite_escape_string($this->post_attr['proxyPort']));
			$ba->set('proxyType', sqlite_escape_string($this->post_attr['proxyType']));
			$ba->set('proxyUser', sqlite_escape_string($this->post_attr['proxyUser']));
			if (isset($this->post_attr['proxySecret'])) {
				$ba->set('proxySecret', sqlite_escape_string($this->post_attr['proxySecret'])); // TODO: how to update the code to hash the password when a customer had already set a password ?
			}
			$ba = new beroAri();
			$table .= Helper::activate(1, true, $this->m_attr, ACTIVATE);
			return $table;
		}
		else if ($this->post_attr['proxyEnabled'] == '0') {
			$ba->set('proxyEnabled', 0);
			$ba->set('proxyAddress', "");
			$ba->set('proxyPort', "");
			$ba->set('proxyType', "");
			$ba->set('proxyUser', "");
			$ba->set('proxySecret', "");
			$ba = new beroAri();
			$table .= Helper::activate(1, true, $this->m_attr, ACTIVATE);
			return $table;
		}

		
		//Disconnect remote access for support
		if (isset($this->post_attr['disconnect_submit'])) {
			exec ("ps aux|grep ssh|grep StrictHostKeyChecking=no|grep -v grep|awk '{print $1}'|head -1", $output, $retval);
			if (sizeof ($output)) {
				exec ("kill -9 " . $output[0]);
			}
			
			$ba->delete ("delete from key_val where key='permanent_remote_support_cmd'");
			$ba->delete ("delete from key_val where key='permanent_remote_support_key'");
			$ba->set ("permanent_remote_support_active", "0");
			if (file_exists ("/usr/conf/remotesupport.conf")) {
				unlink ("/usr/conf/remotesupport.conf");
			}
			return $table;
		}
		
		//Remote access for support
		if (isset($this->post_attr['supportKey'])) {
			$apiUrl = 'https://' . $ba->get("cloudAddress") . '/index.php?r=api/support-access&key=' 
				. $this->post_attr['supportKey'] . '&serial=' . $serial;
			if (($bc->get('root', 'lan-nameserver') == "" || $bc->get('root', 'lan-gateway') == "") && $bc->get('root', 'lan-gateway') == "no") {
				$table .= "<div class='text-center'><img src='./includes/images/punkt_rot.gif'><font color='red'>" . CLOUD_NETWORK_HINT . "</font><br/></div>";
			} else if (!$this->isDomainAvailable($ba->get("cloudAddress"))) {
				$table .= "<div class='text-center'><img src='./includes/images/punkt_rot.gif'><font color='red'>" . CLOUD_NO_CONNECTION_HINT . "</font><br/></div>";
			} else {
				$ch = curl_init();


				curl_setopt($ch, CURLOPT_URL, $apiUrl);
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
				if (! $result = curl_exec($ch)) {
					$table.= "<div class='text-center'><img src='./includes/images/punkt_rot.gif'><font color='red'>" . curl_error($ch) . "</font><br/></div>";
				}
				curl_close($ch);
				if ($result != "" && trim($result)== "Invalid data") {
					$table .= "<div class='text-center'><img src='./includes/images/punkt_rot.gif'><font color='red'>" . $result . "</font><br/></div>";
				}
				else {
					$result = unserialize($result);
					
					if (isset($result['sshKey']) && isset($result['sysCmd'])) {
						file_put_contents('/tmp/key', $result['sshKey']);
						chmod('/tmp/key', 0600);
						
						$ba->set ("permanent_remote_support_active", "0");
						$ba->set ("permanent_remote_support_key", $result['sshKey']);
						$ba->set ("permanent_remote_support_cmd", $result['sysCmd']);
						
						//kill any running instance
						exec ("ps aux|grep ssh|grep StrictHostKeyChecking=no|grep -v grep|awk '{print $1}'|head -1", $output, $retval);
						if (sizeof ($output)) {
							exec ("kill -9 " . $output[0]);
						}
						exec('/sbin/' . $result['sysCmd']);
						
						if (isset($this->post_attr['supportPermanent'])) {
							$ba->set ("permanent_remote_support_active", "1");
							//since we don't do an activate, write the conf file here
							file_put_contents("/usr/conf/remotesupport.conf", "SSHCMD:" . $result['sysCmd'] . "\n");
							file_put_contents("/usr/conf/remotesupport.conf", $result['sshKey'] . "\n", FILE_APPEND);
						}
						else if (file_exists ("/usr/conf/remotesupport.conf")) {
							unlink ("/usr/conf/remotesupport.conf");
						}
						
						$table .= "<div class='text-center'><img src='./includes/images/punkt_gruen.gif'><font color='black'> " . CLOUD_SUPPORT_STARTED_MESSAGE . "<br/></div>";
					}
					else{
						$table .= "<div class='text-center'><img src='./includes/images/punkt_rot.gif'><font color='red'>Invalid request</font><br/></div>";
					}
				}
			}
			return $table;
		}

		return $table;
	}
}
?>
